@@ -1,3 +1,4 @@
+import sys
 from awsjuju.common import Unit, BaseController
 
 
@@ -6,18 +7,6 @@ class Controller(BaseController):
     def __init__(self, unit=None):
         self.unit = unit or Unit()
 
-    def on_joined(self):
-        tags = self.get_tags()
-        ec2 = self.get_ec2()
-
-        instance = self.get_instance(ec2)
-        tags['Name'] = self.unit.remote_unit
-        self.update_tags(instance, tags)
-
-        instance = self.get_bootstrap(ec2)
-        tags['Name'] = 'juju-state-server'
-        self.update_tags(instance, tags)
-
     def get_tags(self):
         tags = {}
         config = self.unit.config_get()
@@ -28,6 +17,9 @@ class Controller(BaseController):
         return tags
 
     def get_bootstrap(self, ec2):
+        # TODO this seems to assume api stability around groups, should play nice
+        # with either group. This also is our only ability at the moment to retrieve
+        # the environment name.
         groups = [g.strip() for g in
                   self.ec2_metadata['security-groups'].split('\n')]
         groups.sort()
@@ -37,20 +29,35 @@ class Controller(BaseController):
             'instance.group': env_group})
         return result[0].instances[0]
 
-    def update_tags(self, instance, tags):
-        # be parsiminous as each tag is a roundtrip.
+    def update_tags(self, instance, tags, only_unset=True):
+        # be parsiminous as each tag modification is a roundtrip.
         updates = {}
         for k, v in tags.items():
+            if only_unset and k in tags:
+                continue
             if instance.tags.get(k) != v:
                 updates[k] = v
 
         for k, v in updates.items():
             instance.add_tag(k, v)
 
+    def on_joined(self):
+        tags = self.get_tags()
+        ec2 = self.get_ec2()
+
+        instance = self.get_instance(ec2)
+        tags['Name'] = self.unit.remote_unit
+        tags['juju-env'] =  self.unit.env_uuid
+        self.update_tags(instance, tags, only_unset=True)
+
+        instance = self.get_bootstrap(ec2)
+        tags['Name'] = 'juju-state-server-%s' % self.unit.env_uuid
+        tags['juju-env'] =  self.unit.env_uuid        
+        self.update_tags(instance, tags, only_unset=True)
+
 
 def main():
-    pass
-
+    Controller.main(sys.argv[1])
 
 if __name__ == '__main__':
     main()
